/* ========================================
 *
 * Copyright YOUR COMPANY, THE YEAR
 * All Rights Reserved
 * UNPUBLISHED, LICENSED SOFTWARE.
 *
 * CONFIDENTIAL AND PROPRIETARY INFORMATION
 * WHICH IS THE PROPERTY OF your company.
 *
 * ========================================
*/


#include "mxd_betacy_sdk.h"
    
#include <project.h>
#include "CyBLE_MTK.h"
#include "MTK_CI.h"
#include "stdio.h"
#include "stdbool.h"
#include "ble_hal.h"
//#include <cy_ble_stack.h>


static uint32_t MTK_packet_length, MTK_packet_type;
static enum UART_cmds command = NO_CMD, prev_command;
static int32_t cmd_args[MAX_CMD_ARG_LEN], prev_cmd_args[MAX_CMD_ARG_LEN];
static bool DTM_test_inprogress = false;
static bool rx_count_read = false;

static uint32_t msCount, time_elapsed;
static int32 i32rssi=-90;
static int32_t DTM_TX_end_delay;
static uint16 rx_count = 0;
static volatile bool new_cmd_flag;
static uint32_t msCount_backup;
static bool adv_enable = false, adv_trig = false;
static cy_stc_ble_mxd_pa_cal_param_t paCalConfig;
void betacyFdCal();

/************************************************************************************************
 *
 *
 ************************************************************************************************/
void MTK_init()
{
    MTK_packet_length = DEFAULT_MTK_PACKET_LENGTH;
    MTK_packet_type = DEFAULT_MTK_PACKET_TYPE;
    new_cmd_flag = false;
    DTM_TX_end_delay = DEFAULT_DTM_TX_END_DELAY;
    MTK_CI_start();
}


void UART_print_int(int32 input)
{
    char temp[20];
    int32_t i = 0;

    temp[i] = '\0';
    sprintf(temp, "%ld", input);
    while (temp[i] != '\0')
    {
        MTK_UartPutChar(temp[i++]);
    }
}

void UART_print_uint(uint32 input)
{
    char temp[20];
    int32_t i = 0;

    temp[i] = '\0';
    sprintf(temp, "%lu", input);
    while (temp[i] != '\0')
    {
        MTK_UartPutChar(temp[i++]);
    }
}

void UART_print_hex(uint32 input)
{
	char temp[20];
	int32_t i = 0;
    temp[i] = '\0';
    sprintf(temp, "%x", (unsigned int) input);
    while (temp[i] != '\0')
    {
        MTK_UartPutChar(temp[i++]);
    }
}

/************************************************************************************************
 *
 *
 ************************************************************************************************/
static void backup_cmd_args()
{
    prev_command = command;
    prev_cmd_args[0] = cmd_args[0];
    prev_cmd_args[1] = cmd_args[1];
    prev_cmd_args[2] = cmd_args[2];
}

/************************************************************************************************
 *
 *
 ************************************************************************************************/
static void print_test_status()
{
    if (prev_command == TXP)
    {
        int32_t temp = 0;
        UART_print_int(temp);
        MTK_UartPutString("\r\n");
    }
    else if (prev_command == RXP)
    {
        UART_print_int(rx_count);
        MTK_UartPutString("\r\n");
    }
    else if (prev_command == SPL)
    {
        UART_print_int(get_packet_length());
        MTK_UartPutString("\r\n");
    }
    else if (prev_command == SPT)
    {
        UART_print_int(get_packet_type());
        MTK_UartPutString("\r\n");
    }
    else if (prev_command == TXC)
    {
        UART_print_int(msCount_backup);
        MTK_UartPutString("\r\n");
    }
    else if (prev_command == RSI)
    {
        UART_print_int(i32rssi);
        MTK_UartPutString(" dBm\r\n");
    }
}


static void execute_RRS()
{
    if (prev_command == TXP)
    {
        ll_hal_stop_dtm();
        DTM_test_inprogress = false;
    }
    else if (prev_command == RXP)
    {
        if (DTM_test_inprogress == true)
        {
            rx_count = ll_hal_stop_dtm();
            DTM_test_inprogress = false;
        }
    }
    else if ((prev_command == TXC) || (prev_command == TXM))
    {
        ll_hal_stop_dtm();
        if ((prev_cmd_args[2] >= 0) && (msCount > 0))
        {
            msCount_backup = msCount;
            msCount_backup = prev_cmd_args[2] - msCount_backup;
        }
        else if (prev_cmd_args[2] == -1)
        {
            msCount_backup = time_elapsed;
            DTM_test_inprogress = false;
        }
        else if ((prev_cmd_args[2] >= 0) && (msCount == 0))
        {
            msCount_backup = prev_cmd_args[2] - msCount_backup;
        }
        
    }
    print_test_status();
}

/************************************************************************************************
 *
 *
 ************************************************************************************************/
extern void betacyModeGainInit(uint32 manual_enable, uint32 freq);
extern void betacyModeGainInit_dbg(uint32 manual_enable, uint16 freq);
extern void betacypac_adc_std_db();
extern void betacypac_std_dbg(uint16 chnl, uint16 target_gain);//_3, uint16 target_2 ,uint16 target_1,uint16 target_0);
extern void betacypacOne_cal_dbg(uint16 chnl, uint16 target_gain);//3, uint16 target_2 ,uint16 target_1,uint16 target_0);
extern void betacypacFour_cal_dbg(uint16 chnl, uint16 target_3, uint16 target_2 ,uint16 target_1,uint16 target_0);
extern void betacyLoLock(uint16 trx, uint16 freqMHz);

void rf_reg_read(uint16 regAddr, uint16 * pRegVal);
void ProcessMTKCommands()
{

    int32_t args_length = 0;

    command = CI_get_command(cmd_args, &args_length);
    if(command != NO_CMD)
    {
        if ((DTM_test_inprogress == true) && ((command == TXP) || (command == RXP) || (command == TXC)))
        {
            execute_RRS();
        }
        switch(command)
        {
            case TXP:
            {
                backup_cmd_args();
                DTM_tx_start(cmd_args[0], cmd_args[1], cmd_args[2]);
                break;
            }   
            case RXP:
            {
                backup_cmd_args();
                DTM_rx_start(cmd_args[0]);
                break;
            }   
            case RRS:
            {
                execute_RRS();
                break;
            }
            case PST:
            {
                print_test_status();
                break;
            }
            case SPL:
            {
                backup_cmd_args();
                set_packet_length(cmd_args[0]);
                break;
            }
            case SPT:
            {
                backup_cmd_args();
                set_packet_type(cmd_args[0]);
                break;
            }
            case TXC:
            {
                backup_cmd_args();
                CyBleHal_SetTxPower(ll_hal_translate_power(cmd_args[1]));
                ll_hal_transmit_carrier_wave(cmd_args[0]);
                time_elapsed = 0;
                if (cmd_args[2] > -1)
                {
                    CyDelay(cmd_args[2]);
                }
                else
                {
                    CyDelay(0);
                }
                break;
            }
            case ADV:
            {
                backup_cmd_args();
                Start_Adv_Emulation();
                break;
            }
            case CAP:
            {
                ll_hal_set_captrim(cmd_args[0]);
                break;
            }
            case SRP:
            {
                CyBleHal_SetDataRate(cmd_args[0]-1);
                break;
            }
            case RRR:
            {
                ll_hal_stop_dtm();
                uint16 temp;
                if((cmd_args[0]<0x1000)||(cmd_args[0]>0x1100))
                {
                    temp = CyBleHal_SingleCPU_RCBRead(cmd_args[0]);   
                }
                else
                {
                    CyBleHal_Enable_RCB_Access();
                    rf_reg_read(cmd_args[0], &temp);
                    CyBleHal_Disable_RCB_Access();
                }
                UART_print_hex(temp);
                //UART_print_int(temp);
                MTK_UartPutString("\r\n");
                break;
            }
            case WRR:
            {
                CyBleHal_SingleCPU_RCBWrite(cmd_args[0],cmd_args[1]);
                break;
            }
            case RPR:
            {
                uint32 temp;
                temp = CY_GET_REG32(cmd_args[0]);
                UART_print_uint(temp);
                MTK_UartPutString("\r\n");
                break;
            }
            case WPR:
            {
                CY_SET_REG32(cmd_args[0], cmd_args[1]);
                break;
            }
            case TXM:
            {
                backup_cmd_args();
                CyBleHal_SetTxPower(ll_hal_translate_power(cmd_args[1]));
                ll_hal_tx_modulated(cmd_args[0], MTK_packet_type);
                time_elapsed = 0;
                if (cmd_args[2] > -1)
                {
                    CyDelay(cmd_args[2]);
                }
                else
                {
                    CyDelay(0);
                }                
                break;
            }
            case CAL:
            {
                if (cmd_args[0]!=0)
                {
                    CyBleHal_Enable_RCB_Access();
                    if(cmd_args[0] & FDPOUT_CAL_MASK)
                    {
                        for (uint8 i=0; i<7; i++)
                            paCalConfig.targetTable[i] = cmd_args[1];
                        betacyFdPoutInit(0, paCalConfig);
                        //MTK_UartPutString("FdPout Calibration Done\n\r");
                    }
                    else if(cmd_args[0] & FD_CAL_MASK)
                    {
                        betacyFdCal();
                        MTK_UartPutString("Fd Calibration Done\n\r");
                    }
                    if (cmd_args[0] & DCOC_CAL_MASK)
                    {
                        betacyDcocCali();
                        MTK_UartPutString("Dcoc Calibration Done\n\r");
                    }
                    if(cmd_args[0] & MODEGAIN_CAL_MASK)
                    {

                        _REGBS(clk_clk_en_reg, _BIT0);
                        betacyModeGainInit(1, 2440);
                        MTK_UartPutString("Modegain Calibration Done\n\r");
                    }
                    rf_reg_write(pmu_mode_transition_reg, 0x0140);//tuner_rbus_write('1e02', '1140');
                    rf_reg_write(pmu_mode_transition_reg, 0x1940);//tuner_rbus_write('1e02', '1940');
                    CyBleHal_Disable_RCB_Access();  
                }
                else
                {
                    MTK_UartPutString("No Calibration Done\n\r");
                }
                break;
            }
            case PCA:
            {
                CyBleHal_Enable_RCB_Access();  
                for (uint8 i=0; i<7; i++)
                    paCalConfig.targetTable[i] = cmd_args[0];
                paCalConfig.lobuffLimit = cmd_args[1];
                paCalConfig.calValLimit = cmd_args[2];
                betacyFdPoutInit(0, paCalConfig);
                rf_reg_write(pmu_mode_transition_reg, 0x0140);//tuner_rbus_write('1e02', '1140');
                rf_reg_write(pmu_mode_transition_reg, 0x1940);//tuner_rbus_write('1e02', '1940');
                CyBleHal_Disable_RCB_Access();  
                break;
            }
            case RSI:
            {
                uint32 temp;
                backup_cmd_args();
                ll_hal_stop_dtm();
                if (cmd_args[0]==0)
                {
                    temp = CyBleHal_GetRSSIfromRadio();
                    PrintRSSI(temp);

                }
                else if (cmd_args[0]==1)
                {
                    //temp = CY_GET_REG32(CYREG_BLE_BLESS_LL_PKT_RSSI_CH_ENERGY);
                    //UART_print_uint((temp&0xffff)>>14);
                    //MTK_UartPutString(" ,");
                    //UART_print_uint(temp&0x3fff);
                    //MTK_UartPutString(" ,");
                    temp = CyBleHal_GetRSSIfromLinkLayer();
                    PrintRSSI(temp);
                    
                }
                else if (cmd_args[0]==2)
                {//Channel Energy Mode: RSI 2 CH
                    BLE->BLELL.LL_CONFIG |= BLE_BLELL_LL_CONFIG_RSSI_ENERGY_RD_Msk;
                    CyBleHal_Enable_Single_CPU_ReadWrite_in_TXRX();//(1) Enable Single CPU read writes during active TXRX
                    //CyBleHal_SingleCPU_RCBWrite( 0x845,0x1 );//Update RSSI in radio Reg
                    DTM_rx_start(cmd_args[1]);//select rx channel
                    CyDelay(10);                    

                    if (cmd_args[2] == 0)
                    {//LL register read Mode
                        CyBleHal_SingleCPU_RCBWrite( 0x845,0x1 );                       
                        CyBleHal_SingleCPU_RCBWrite( 0x845,0x0 );//Update RSSI in radio Reg
                        CyDelayUs(100);
                        temp = BLE->BLESS.LL_PKT_RSSI_CH_ENERGY;
                        //temp = BLE->BLESS.LL_PKT_RSSI_CH_ENERGY;
                        MTK_UartPutString("Channel:");
                        UART_print_uint((temp>>16)&0x1f);
                        MTK_UartPutString(" ,");
                        if((temp&(0x1<<22)))
                        {
                            MTK_UartPutString(" This is packet RSSI: ");
                        }
                        else
                        {
                            MTK_UartPutString("This is Channel Energy:");
                        }
                        PrintRSSI(temp);
                    }
                    else if (cmd_args[2] == 1)
                    {
                       
                        CyBleHal_SingleCPU_RCBWrite( 0x845,0x1 );                       
                        CyBleHal_SingleCPU_RCBWrite( 0x845,0x0 );//Update RSSI in radio Reg
                        CyDelayUs(100);
                        temp = CyBleHal_SingleCPU_RCBRead( 0x0a03 );
                        PrintRSSI(temp);

                        
                    }
                    else if (cmd_args[2] == 2)
                    {
                        temp = CyBleHal_SingleCPU_RCBRead( 0x0a04 );
                        PrintRSSI(temp);
                    }

                }
                else if (cmd_args[0] ==3)
                {//WEIY method no toggle
                    volatile uint32 u32tempo;
                    //BLE->BLELL.LL_CONFIG |= BLE_BLELL_LL_CONFIG_RSSI_ENERGY_RD_Msk;              
                    u32tempo = CyBleHal_SingleCPU_RCBRead(0x1e02);
                    u32tempo &= ~(0x3f);//clear channel field
                    u32tempo |= ((0x3f)&cmd_args[1]);//channel number in [5:0]
                    if (cmd_args[2]-1 == 0)
                    {u32tempo &= ~(0x1<<10);}//RX Data rate (1MBPS =1,2MBPS=2)
                    else
                    u32tempo |= 0x1<<10;
                    u32tempo |= 0x1<<14;//rx_en
                    CyBleHal_SingleCPU_RCBWrite(0x1e02,u32tempo);
                    CyDelayUs(250);
                    CyBleHal_SingleCPU_RCBWrite( 0x845,0x1 );
                    CyDelayUs(100);
                    u32tempo &= ~(0x1<<14);
                    CyBleHal_SingleCPU_RCBWrite(0x1e02,u32tempo);//disable_rx                    
                    temp=CyBleHal_SingleCPU_RCBRead(0xa03);
                    UART_print_uint((temp>>14)&0x3);//0xa03[15:14]
                    MTK_UartPutString(" ,");
                    UART_print_uint((temp)&0x3fff);//0xa03[13:0]
                    MTK_UartPutString(" ,");
                    temp=CyBleHal_SingleCPU_RCBRead(0xa04);
                    UART_print_uint((temp>>13)&0x3);//0xa03[14:13]
                    MTK_UartPutString(" ,");
                    UART_print_uint((temp)&0x1fff);//0xa03[12:0] 
                    MTK_UartPutString("\r\n");
                }
                else if (cmd_args[0] ==4)
                {//WEIY method
                    volatile uint32 u32tempo;
                    //BLE->BLELL.LL_CONFIG |= BLE_BLELL_LL_CONFIG_RSSI_ENERGY_RD_Msk;              
                    u32tempo = CyBleHal_SingleCPU_RCBRead(0x1e02);
                    u32tempo &= ~(0x3f);//clear channel field
                    u32tempo |= ((0x3f)&cmd_args[1]);//channel number in [5:0]
                    u32tempo |= (0x1&(cmd_args[2]-1))<<10;//RX Data rate (1MBPS =1,2MBPS=2)
                    u32tempo |= 0x1<<14;//rx_en
                    CyBleHal_SingleCPU_RCBWrite(0x1e02,u32tempo);
                    CyDelayUs(250);
                    CyBleHal_SingleCPU_RCBWrite( 0x845,0x1 );
                    
                    CyBleHal_SingleCPU_RCBWrite( 0x845,0x0 );
                    CyDelayUs(100);
                    u32tempo &= ~(0x1<<14);
                    CyBleHal_SingleCPU_RCBWrite(0x1e02,u32tempo);//disable_rx                    
                    temp=CyBleHal_SingleCPU_RCBRead(0xa03);
                    UART_print_uint((temp>>14)&0x3);//0xa03[15:14]
                    MTK_UartPutString(" ,");
                    UART_print_uint((temp)&0x3fff);//0xa03[13:0]
                    MTK_UartPutString(" ,");
                    temp=CyBleHal_SingleCPU_RCBRead(0xa04);
                    UART_print_uint((temp>>13)&0x3);//0xa03[14:13]
                    MTK_UartPutString(" ,");
                    UART_print_uint((temp)&0x1fff);//0xa03[12:0] 
                    MTK_UartPutString("\r\n");
                }
                //print_test_status();
                break;
            }
			

            default:
            {
                execute_RRS();
                break;
            }
        }
    }
    if((adv_enable == true) && (adv_trig == true))
    {
        DTM_tx_adv_trig(cmd_args[0], cmd_args[1]);
    }
}

/************************************************************************************************
 *
 *
 ************************************************************************************************/
void PrintRSSI(uint32 temp)
{
    ll_hal_stop_dtm();
    UART_print_uint((temp&0xffff)>>14);
    MTK_UartPutString(" ,");
    UART_print_uint(temp&0x3fff);
    MTK_UartPutString(" ,");
    i32rssi = CyBleHal_GetRSSIfromRegval(temp);
    UART_print_int(i32rssi);
    MTK_UartPutString(" dBm\r\n");
}
/************************************************************************************************
 *
 *
 ************************************************************************************************/
void set_packet_length(uint32_t length)
{
    MTK_packet_length = length;
}

/************************************************************************************************
 *
 *
 ************************************************************************************************/
void set_packet_type(uint32_t type)
{
    MTK_packet_type = type;
}

/************************************************************************************************
 *
 *
 ************************************************************************************************/
uint32_t get_packet_length()
{
    return MTK_packet_length;
}

/************************************************************************************************
 *
 *
 ************************************************************************************************/
uint32_t get_packet_type()
{
    return MTK_packet_type;
}

/************************************************************************************************
 *
 *
 ************************************************************************************************/
void DTM_tx_start(uint32_t channel, int8 power, int32_t num_packets)
{
    ll_hal_stop_dtm();
    DTM_test_inprogress = false;
    CyBleHal_SetTxPower(ll_hal_translate_power(power));
    ll_hal_dtm_tx_test(MTK_packet_length, MTK_packet_type, channel);
    if (num_packets>=0)
    {
        CyDelayUs(num_packets*UNIT_PACKET_TX_TIMEOUT);
        ll_hal_stop_dtm();
        DTM_test_inprogress = false;
    }
    else
    {
        DTM_test_inprogress = true;
    }
}

void Start_Adv_Emulation()
{
  // Timer_SetOneShot(1);
   adv_enable = true;
   adv_trig = true;
}

void DTM_tx_adv_trig(uint8 channel, uint16 adv_interval)
{
    adv_trig =  false;
    /*Transmit Single Packet*/
    ll_hal_dtm_tx_test(37, 0, channel);
    CyDelayUs(UNIT_PACKET_TX_TIMEOUT);
    ll_hal_stop_dtm();
}

/************************************************************************************************
 *
 *
 ************************************************************************************************/
void DTM_rx_start(uint32_t channel)
{
    ll_hal_stop_dtm();
    DTM_test_inprogress = false;
    ll_hal_dtm_rx_test(channel);
    DTM_test_inprogress = true;
    rx_count_read = false;
}

/* [] END OF FILE */
