/***************************************************************************//**
* \file MTK.h
* \version 2.0
*
*  This file provides constants and parameter values for the UART component.
*
********************************************************************************
* \copyright
* Copyright 2016-2017, Cypress Semiconductor Corporation. All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#if !defined(MTK_CY_SCB_UART_PDL_H)
#define MTK_CY_SCB_UART_PDL_H

#include "cyfitter.h"
#include "scb/cy_scb_uart.h"

#if defined(__cplusplus)
extern "C" {
#endif

/***************************************
*   Initial Parameter Constants
****************************************/

#define MTK_DIRECTION  (3U)
#define MTK_ENABLE_RTS (0U)
#define MTK_ENABLE_CTS (0U)

/* UART direction enum */
#define MTK_RX    (0x1U)
#define MTK_TX    (0x2U)

#define MTK_ENABLE_RX  (0UL != (MTK_DIRECTION & MTK_RX))
#define MTK_ENABLE_TX  (0UL != (MTK_DIRECTION & MTK_TX))


/***************************************
*        Function Prototypes
***************************************/
/**
* \addtogroup group_general
* @{
*/
/* Component specific functions. */
void MTK_Start(void);

/* Basic functions */
__STATIC_INLINE cy_en_scb_uart_status_t MTK_Init(cy_stc_scb_uart_config_t const *config);
__STATIC_INLINE void MTK_DeInit(void);
__STATIC_INLINE void MTK_Enable(void);
__STATIC_INLINE void MTK_Disable(void);

/* Register callback. */
__STATIC_INLINE void MTK_RegisterCallback(cy_cb_scb_uart_handle_events_t callback);

/* Configuration change. */
#if (MTK_ENABLE_CTS)
__STATIC_INLINE void MTK_EnableCts(void);
__STATIC_INLINE void MTK_DisableCts(void);
#endif /* (MTK_ENABLE_CTS) */

#if (MTK_ENABLE_RTS)
__STATIC_INLINE void     MTK_SetRtsFifoLevel(uint32_t level);
__STATIC_INLINE uint32_t MTK_GetRtsFifoLevel(void);
#endif /* (MTK_ENABLE_RTS) */

__STATIC_INLINE void MTK_EnableSkipStart(void);
__STATIC_INLINE void MTK_DisableSkipStart(void);

#if (MTK_ENABLE_RX)
/* Low level: Receive direction. */
__STATIC_INLINE uint32_t MTK_Get(void);
__STATIC_INLINE uint32_t MTK_GetArray(void *buffer, uint32_t size);
__STATIC_INLINE void     MTK_GetArrayBlocking(void *buffer, uint32_t size);
__STATIC_INLINE uint32_t MTK_GetRxFifoStatus(void);
__STATIC_INLINE void     MTK_ClearRxFifoStatus(uint32_t clearMask);
__STATIC_INLINE uint32_t MTK_GetNumInRxFifo(void);
__STATIC_INLINE void     MTK_ClearRxFifo(void);
#endif /* (MTK_ENABLE_RX) */

#if (MTK_ENABLE_TX)
/* Low level: Transmit direction. */
__STATIC_INLINE uint32_t MTK_Put(uint32_t data);
__STATIC_INLINE uint32_t MTK_PutArray(void *buffer, uint32_t size);
__STATIC_INLINE void     MTK_PutArrayBlocking(void *buffer, uint32_t size);
__STATIC_INLINE void     MTK_PutString(char_t const string[]);
__STATIC_INLINE void     MTK_SendBreakBlocking(uint32_t breakWidth);
__STATIC_INLINE uint32_t MTK_GetTxFifoStatus(void);
__STATIC_INLINE void     MTK_ClearTxFifoStatus(uint32_t clearMask);
__STATIC_INLINE uint32_t MTK_GetNumInTxFifo(void);
__STATIC_INLINE bool     MTK_IsTxComplete(void);
__STATIC_INLINE void     MTK_ClearTxFifo(void);
#endif /* (MTK_ENABLE_TX) */

#if (MTK_ENABLE_RX)
/* High level: Ring buffer functions. */
__STATIC_INLINE void     MTK_StartRingBuffer(void *buffer, uint32_t size);
__STATIC_INLINE void     MTK_StopRingBuffer(void);
__STATIC_INLINE void     MTK_ClearRingBuffer(void);
__STATIC_INLINE uint32_t MTK_GetNumInRingBuffer(void);

/* High level: Receive direction functions. */
__STATIC_INLINE cy_en_scb_uart_status_t MTK_Receive(void *buffer, uint32_t size);
__STATIC_INLINE void     MTK_AbortReceive(void);
__STATIC_INLINE uint32_t MTK_GetReceiveStatus(void);
__STATIC_INLINE uint32_t MTK_GetNumReceived(void);
#endif /* (MTK_ENABLE_RX) */

#if (MTK_ENABLE_TX)
/* High level: Transmit direction functions. */
__STATIC_INLINE cy_en_scb_uart_status_t MTK_Transmit(void *buffer, uint32_t size);
__STATIC_INLINE void     MTK_AbortTransmit(void);
__STATIC_INLINE uint32_t MTK_GetTransmitStatus(void);
__STATIC_INLINE uint32_t MTK_GetNumLeftToTransmit(void);
#endif /* (MTK_ENABLE_TX) */

/* Interrupt handler */
__STATIC_INLINE void MTK_Interrupt(void);
/** @} group_general */


/***************************************
*    Variables with External Linkage
***************************************/
/**
* \addtogroup group_globals
* @{
*/
extern uint8_t MTK_initVar;
extern cy_stc_scb_uart_config_t const MTK_config;
extern cy_stc_scb_uart_context_t MTK_context;
/** @} group_globals */


/***************************************
*         Preprocessor Macros
***************************************/
/**
* \addtogroup group_macros
* @{
*/
/** The pointer to the base address of the hardware */
#define MTK_HW     ((CySCB_Type *) MTK_SCB__HW)
/** @} group_macros */


/***************************************
*    In-line Function Implementation
***************************************/

/*******************************************************************************
* Function Name: MTK_Init
****************************************************************************//**
*
* Invokes the Cy_SCB_UART_Init() PDL driver function.
*
*******************************************************************************/
__STATIC_INLINE cy_en_scb_uart_status_t MTK_Init(cy_stc_scb_uart_config_t const *config)
{
   return Cy_SCB_UART_Init(MTK_HW, config, &MTK_context);
}


/*******************************************************************************
* Function Name: MTK_DeInit
****************************************************************************//**
*
* Invokes the Cy_SCB_UART_DeInit() PDL driver function.
*
*******************************************************************************/
__STATIC_INLINE void MTK_DeInit(void)
{
    Cy_SCB_UART_DeInit(MTK_HW);
}


/*******************************************************************************
* Function Name: MTK_Enable
****************************************************************************//**
*
* Invokes the Cy_SCB_UART_Enable() PDL driver function.
*
*******************************************************************************/
__STATIC_INLINE void MTK_Enable(void)
{
    Cy_SCB_UART_Enable(MTK_HW);
}


/*******************************************************************************
* Function Name: MTK_Disable
****************************************************************************//**
*
* Invokes the Cy_SCB_UART_Disable() PDL driver function.
*
*******************************************************************************/
__STATIC_INLINE void MTK_Disable(void)
{
    Cy_SCB_UART_Disable(MTK_HW, &MTK_context);
}


/*******************************************************************************
* Function Name: MTK_RegisterCallback
****************************************************************************//**
*
* Invokes the Cy_SCB_UART_RegisterCallback() PDL driver function.
*
*******************************************************************************/
__STATIC_INLINE void MTK_RegisterCallback(cy_cb_scb_uart_handle_events_t callback)
{
    Cy_SCB_UART_RegisterCallback(MTK_HW, callback, &MTK_context);
}


#if (MTK_ENABLE_CTS)
/*******************************************************************************
* Function Name: MTK_EnableCts
****************************************************************************//**
*
* Invokes the Cy_SCB_UART_EnableCts() PDL driver function.
*
*******************************************************************************/
__STATIC_INLINE void MTK_EnableCts(void)
{
    Cy_SCB_UART_EnableCts(MTK_HW);
}


/*******************************************************************************
* Function Name: Cy_SCB_UART_DisableCts
****************************************************************************//**
*
* Invokes the Cy_SCB_UART_DisableCts() PDL driver function.
*
*******************************************************************************/
__STATIC_INLINE void MTK_DisableCts(void)
{
    Cy_SCB_UART_DisableCts(MTK_HW);
}
#endif /* (MTK_ENABLE_CTS) */


#if (MTK_ENABLE_RTS)
/*******************************************************************************
* Function Name: MTK_SetRtsFifoLevel
****************************************************************************//**
*
* Invokes the Cy_SCB_UART_SetRtsFifoLevel() PDL driver function.
*
*******************************************************************************/
__STATIC_INLINE void MTK_SetRtsFifoLevel(uint32_t level)
{
    Cy_SCB_UART_SetRtsFifoLevel(MTK_HW, level);
}


/*******************************************************************************
* Function Name: MTK_GetRtsFifoLevel
****************************************************************************//**
*
* Invokes the Cy_SCB_UART_GetRtsFifoLevel() PDL driver function.
*
*******************************************************************************/
__STATIC_INLINE uint32_t MTK_GetRtsFifoLevel(void)
{
    return Cy_SCB_UART_GetRtsFifoLevel(MTK_HW);
}
#endif /* (MTK_ENABLE_RTS) */


/*******************************************************************************
* Function Name: MTK_EnableSkipStart
****************************************************************************//**
*
* Invokes the Cy_SCB_UART_EnableSkipStart() PDL driver function.
*
*******************************************************************************/
__STATIC_INLINE void MTK_EnableSkipStart(void)
{
    Cy_SCB_UART_EnableSkipStart(MTK_HW);
}


/*******************************************************************************
* Function Name: MTK_DisableSkipStart
****************************************************************************//**
*
* Invokes the Cy_SCB_UART_DisableSkipStart() PDL driver function.
*
*******************************************************************************/
__STATIC_INLINE void MTK_DisableSkipStart(void)
{
    Cy_SCB_UART_DisableSkipStart(MTK_HW);
}


#if (MTK_ENABLE_RX)
/*******************************************************************************
* Function Name: MTK_Get
****************************************************************************//**
*
* Invokes the Cy_SCB_UART_Get() PDL driver function.
*
*******************************************************************************/
__STATIC_INLINE uint32_t MTK_Get(void)
{
    return Cy_SCB_UART_Get(MTK_HW);
}


/*******************************************************************************
* Function Name: MTK_GetArray
****************************************************************************//**
*
* Invokes the Cy_SCB_UART_GetArray() PDL driver function.
*
*******************************************************************************/
__STATIC_INLINE uint32_t MTK_GetArray(void *buffer, uint32_t size)
{
    return Cy_SCB_UART_GetArray(MTK_HW, buffer, size);
}


/*******************************************************************************
* Function Name: MTK_GetArrayBlocking
****************************************************************************//**
*
* Invokes the Cy_SCB_UART_GetArrayBlocking() PDL driver function.
*
*******************************************************************************/
__STATIC_INLINE void MTK_GetArrayBlocking(void *buffer, uint32_t size)
{
    Cy_SCB_UART_GetArrayBlocking(MTK_HW, buffer, size);
}


/*******************************************************************************
* Function Name: MTK_GetRxFifoStatus
****************************************************************************//**
*
* Invokes the Cy_SCB_UART_GetRxFifoStatus() PDL driver function.
*
*******************************************************************************/
__STATIC_INLINE uint32_t MTK_GetRxFifoStatus(void)
{
    return Cy_SCB_UART_GetRxFifoStatus(MTK_HW);
}


/*******************************************************************************
* Function Name: MTK_ClearRxFifoStatus
****************************************************************************//**
*
* Invokes the Cy_SCB_UART_ClearRxFifoStatus() PDL driver function.
*
*******************************************************************************/
__STATIC_INLINE void MTK_ClearRxFifoStatus(uint32_t clearMask)
{
    Cy_SCB_UART_ClearRxFifoStatus(MTK_HW, clearMask);
}


/*******************************************************************************
* Function Name: MTK_GetNumInRxFifo
****************************************************************************//**
*
* Invokes the Cy_SCB_UART_GetNumInRxFifo() PDL driver function.
*
*******************************************************************************/
__STATIC_INLINE uint32_t MTK_GetNumInRxFifo(void)
{
    return Cy_SCB_UART_GetNumInRxFifo(MTK_HW);
}


/*******************************************************************************
* Function Name: MTK_ClearRxFifo
****************************************************************************//**
*
* Invokes the Cy_SCB_UART_ClearRxFifo() PDL driver function.
*
*******************************************************************************/
__STATIC_INLINE void MTK_ClearRxFifo(void)
{
    Cy_SCB_UART_ClearRxFifo(MTK_HW);
}
#endif /* (MTK_ENABLE_RX) */


#if (MTK_ENABLE_TX)
/*******************************************************************************
* Function Name: MTK_Put
****************************************************************************//**
*
* Invokes the Cy_SCB_UART_Put() PDL driver function.
*
*******************************************************************************/
__STATIC_INLINE uint32_t MTK_Put(uint32_t data)
{
    return Cy_SCB_UART_Put(MTK_HW,data);
}


/*******************************************************************************
* Function Name: MTK_PutArray
****************************************************************************//**
*
* Invokes the Cy_SCB_UART_PutArray() PDL driver function.
*
*******************************************************************************/
__STATIC_INLINE uint32_t MTK_PutArray(void *buffer, uint32_t size)
{
    return Cy_SCB_UART_PutArray(MTK_HW, buffer, size);
}


/*******************************************************************************
* Function Name: MTK_PutArrayBlocking
****************************************************************************//**
*
* Invokes the Cy_SCB_UART_PutArrayBlocking() PDL driver function.
*
*******************************************************************************/
__STATIC_INLINE void MTK_PutArrayBlocking(void *buffer, uint32_t size)
{
    Cy_SCB_UART_PutArrayBlocking(MTK_HW, buffer, size);
}


/*******************************************************************************
* Function Name: MTK_PutString
****************************************************************************//**
*
* Invokes the Cy_SCB_UART_PutString() PDL driver function.
*
*******************************************************************************/
__STATIC_INLINE void MTK_PutString(char_t const string[])
{
    Cy_SCB_UART_PutString(MTK_HW, string);
}


/*******************************************************************************
* Function Name: MTK_SendBreakBlocking
****************************************************************************//**
*
* Invokes the Cy_SCB_UART_SendBreakBlocking() PDL driver function.
*
*******************************************************************************/
__STATIC_INLINE void MTK_SendBreakBlocking(uint32_t breakWidth)
{
    Cy_SCB_UART_SendBreakBlocking(MTK_HW, breakWidth);
}


/*******************************************************************************
* Function Name: MTK_GetTxFifoStatus
****************************************************************************//**
*
* Invokes the Cy_SCB_UART_GetTxFifoStatus() PDL driver function.
*
*******************************************************************************/
__STATIC_INLINE uint32_t MTK_GetTxFifoStatus(void)
{
    return Cy_SCB_UART_GetTxFifoStatus(MTK_HW);
}


/*******************************************************************************
* Function Name: MTK_ClearTxFifoStatus
****************************************************************************//**
*
* Invokes the Cy_SCB_UART_ClearTxFifoStatus() PDL driver function.
*
*******************************************************************************/
__STATIC_INLINE void MTK_ClearTxFifoStatus(uint32_t clearMask)
{
    Cy_SCB_UART_ClearTxFifoStatus(MTK_HW, clearMask);
}


/*******************************************************************************
* Function Name: MTK_GetNumInTxFifo
****************************************************************************//**
*
* Invokes the Cy_SCB_UART_GetNumInTxFifo() PDL driver function.
*
*******************************************************************************/
__STATIC_INLINE uint32_t MTK_GetNumInTxFifo(void)
{
    return Cy_SCB_UART_GetNumInTxFifo(MTK_HW);
}


/*******************************************************************************
* Function Name: MTK_IsTxComplete
****************************************************************************//**
*
* Invokes the Cy_SCB_UART_IsTxComplete() PDL driver function.
*
*******************************************************************************/
__STATIC_INLINE bool MTK_IsTxComplete(void)
{
    return Cy_SCB_UART_IsTxComplete(MTK_HW);
}


/*******************************************************************************
* Function Name: MTK_ClearTxFifo
****************************************************************************//**
*
* Invokes the Cy_SCB_UART_ClearTxFifo() PDL driver function.
*
*******************************************************************************/
__STATIC_INLINE void MTK_ClearTxFifo(void)
{
    Cy_SCB_UART_ClearTxFifo(MTK_HW);
}
#endif /* (MTK_ENABLE_TX) */


#if (MTK_ENABLE_RX)
/*******************************************************************************
* Function Name: MTK_StartRingBuffer
****************************************************************************//**
*
* Invokes the Cy_SCB_UART_StartRingBuffer() PDL driver function.
*
*******************************************************************************/
__STATIC_INLINE void MTK_StartRingBuffer(void *buffer, uint32_t size)
{
    Cy_SCB_UART_StartRingBuffer(MTK_HW, buffer, size, &MTK_context);
}


/*******************************************************************************
* Function Name: MTK_StopRingBuffer
****************************************************************************//**
*
* Invokes the Cy_SCB_UART_StopRingBuffer() PDL driver function.
*
*******************************************************************************/
__STATIC_INLINE void MTK_StopRingBuffer(void)
{
    Cy_SCB_UART_StopRingBuffer(MTK_HW, &MTK_context);
}


/*******************************************************************************
* Function Name: MTK_ClearRingBuffer
****************************************************************************//**
*
* Invokes the Cy_SCB_UART_ClearRingBuffer() PDL driver function.
*
*******************************************************************************/
__STATIC_INLINE void MTK_ClearRingBuffer(void)
{
    Cy_SCB_UART_ClearRingBuffer(MTK_HW, &MTK_context);
}


/*******************************************************************************
* Function Name: MTK_GetNumInRingBuffer
****************************************************************************//**
*
* Invokes the Cy_SCB_UART_GetNumInRingBuffer() PDL driver function.
*
*******************************************************************************/
__STATIC_INLINE uint32_t MTK_GetNumInRingBuffer(void)
{
    return Cy_SCB_UART_GetNumInRingBuffer(MTK_HW, &MTK_context);
}


/*******************************************************************************
* Function Name: MTK_Receive
****************************************************************************//**
*
* Invokes the Cy_SCB_UART_Receive() PDL driver function.
*
*******************************************************************************/
__STATIC_INLINE cy_en_scb_uart_status_t MTK_Receive(void *buffer, uint32_t size)
{
    return Cy_SCB_UART_Receive(MTK_HW, buffer, size, &MTK_context);
}


/*******************************************************************************
* Function Name: MTK_GetReceiveStatus
****************************************************************************//**
*
* Invokes the Cy_SCB_UART_GetReceiveStatus() PDL driver function.
*
*******************************************************************************/
__STATIC_INLINE uint32_t MTK_GetReceiveStatus(void)
{
    return Cy_SCB_UART_GetReceiveStatus(MTK_HW, &MTK_context);
}


/*******************************************************************************
* Function Name: MTK_AbortReceive
****************************************************************************//**
*
* Invokes the Cy_SCB_UART_AbortReceive() PDL driver function.
*
*******************************************************************************/
__STATIC_INLINE void MTK_AbortReceive(void)
{
    Cy_SCB_UART_AbortReceive(MTK_HW, &MTK_context);
}


/*******************************************************************************
* Function Name: MTK_GetNumReceived
****************************************************************************//**
*
* Invokes the Cy_SCB_UART_GetNumReceived() PDL driver function.
*
*******************************************************************************/
__STATIC_INLINE uint32_t MTK_GetNumReceived(void)
{
    return Cy_SCB_UART_GetNumReceived(MTK_HW, &MTK_context);
}
#endif /* (MTK_ENABLE_RX) */


#if (MTK_ENABLE_TX)
/*******************************************************************************
* Function Name: MTK_Transmit
****************************************************************************//**
*
* Invokes the Cy_SCB_UART_Transmit() PDL driver function.
*
*******************************************************************************/
__STATIC_INLINE cy_en_scb_uart_status_t MTK_Transmit(void *buffer, uint32_t size)
{
    return Cy_SCB_UART_Transmit(MTK_HW, buffer, size, &MTK_context);
}


/*******************************************************************************
* Function Name: MTK_GetTransmitStatus
****************************************************************************//**
*
* Invokes the Cy_SCB_UART_GetTransmitStatus() PDL driver function.
*
*******************************************************************************/
__STATIC_INLINE uint32_t MTK_GetTransmitStatus(void)
{
    return Cy_SCB_UART_GetTransmitStatus(MTK_HW, &MTK_context);
}


/*******************************************************************************
* Function Name: MTK_AbortTransmit
****************************************************************************//**
*
* Invokes the Cy_SCB_UART_AbortTransmit() PDL driver function.
*
*******************************************************************************/
__STATIC_INLINE void MTK_AbortTransmit(void)
{
    Cy_SCB_UART_AbortTransmit(MTK_HW, &MTK_context);
}


/*******************************************************************************
* Function Name: MTK_GetNumLeftToTransmit
****************************************************************************//**
*
* Invokes the Cy_SCB_UART_GetNumLeftToTransmit() PDL driver function.
*
*******************************************************************************/
__STATIC_INLINE uint32_t MTK_GetNumLeftToTransmit(void)
{
    return Cy_SCB_UART_GetNumLeftToTransmit(MTK_HW, &MTK_context);
}
#endif /* (MTK_ENABLE_TX) */


/*******************************************************************************
* Function Name: MTK_Interrupt
****************************************************************************//**
*
* Invokes the Cy_SCB_UART_Interrupt() PDL driver function.
*
*******************************************************************************/
__STATIC_INLINE void MTK_Interrupt(void)
{
    Cy_SCB_UART_Interrupt(MTK_HW, &MTK_context);
}

#if defined(__cplusplus)
}
#endif

#endif /* MTK_CY_SCB_UART_PDL_H */


/* [] END OF FILE */
