 
/* ========================================
 *
 * Copyright YOUR COMPANY, THE YEAR
 * All Rights Reserved
 * UNPUBLISHED, LICENSED SOFTWARE.
 *
 * CONFIDENTIAL AND PROPRIETARY INFORMATION
 * WHICH IS THE PROPERTY OF your company.
 *
 * ========================================
*/

#include <project.h>
#include <CyBLE_MTK.h>
#include "MTK_CI.h"
#include "stdbool.h"

static uint8_t CI_state;
static uint8_t rx_cmd, cmd_done_flag;
static int32_t cmd_index, cmd_argument_array[MAX_CMD_ARG_LEN], rx_cmd_len;
static uint8_t rx_data[4] = {'\0', '\0', '\0', '\0'}, negative_arg = 0, rx_data_neg_one = 0;

static const CMD_DEF_T command_lookup[NUMBER_OF_COMMANDS - 1] = 
{
/*       1    2  3    0,0          0,1    0,2     1,0          1,1    1,2    2,0          2,1         2,2*/
    {"TXP", TXP, 3, {{  0,           0,         39}, { -20,         0,           4}, { -1,        1000,      65535}}},
    {"RXP", RXP, 2, {{  0,           0,         39}, {  -1,        1000,     65535}, {  0, ARG_DEF_VAL,          0}}},
    {"RRS", RRS, 0, {{  0, ARG_DEF_VAL,          0}, {   0, ARG_DEF_VAL,         0}, {  0, ARG_DEF_VAL,          0}}},
    {"PST", PST, 0, {{  0, ARG_DEF_VAL,          0}, {   0, ARG_DEF_VAL,         0}, {  0, ARG_DEF_VAL,          0}}},
    {"SPL", SPL, 1, {{  0,         255,        255}, {   0, ARG_DEF_VAL,         0}, {  0, ARG_DEF_VAL,          0}}},
    {"SPT", SPT, 1, {{  0,           0,          8}, {   0, ARG_DEF_VAL,         0}, {  0, ARG_DEF_VAL,          0}}},
    {"TXC", TXC, 3, {{  0,           0,         39}, { -20,           0,         4}, { -1,          -1, 2147483647}}},
    {"ADV", ADV, 2, {{  0,           0,         39}, {  32,          32,     16384}, {  0, ARG_DEF_VAL,          0}}},
    {"CAP", CAP, 1, {{  0,          42,        255}, {   0, ARG_DEF_VAL,         0}, {  0, ARG_DEF_VAL,          0}}},
    {"SRP", SRP, 1, {{  1,           1,          2}, {   0, ARG_DEF_VAL,         0}, {  0, ARG_DEF_VAL,          0}}},
    {"RRR", RRR, 1, {{  0,      0x1e02,     0xFFFF}, {   0, ARG_DEF_VAL,         0}, {  0, ARG_DEF_VAL,          0}}},
    {"WRR", WRR, 2, {{  0,      0x1e02,     0xFFFF}, {   0,      0x1800,    0xFFFF}, {  0, ARG_DEF_VAL,          0}}},
    {"RPR", RPR, 1, {{  0,  0x403c1174, 0x7FFFFFFF}, {   0, ARG_DEF_VAL,         0}, {  0, ARG_DEF_VAL,          0}}},
    {"WPR", WPR, 2, {{  0,  0x403c1174, 0x7FFFFFFF}, {   0,        0x48, 0xFFFFFFF}, {  0, ARG_DEF_VAL,          0}}},
    {"TXM", TXM, 3, {{  0,           0,         39}, { -20,           0,         4}, { -1,          -1, 2147483647}}},
	{"RSI", RSI, 3, {{  0,           0,          4}, {   0,           0,        39}, { 0,           0,           2}}},
    {"DBG", DBG, 1, {{  0,           0,         31}, {   0, ARG_DEF_VAL,         0}, {  0, ARG_DEF_VAL,          0}}},
    {"CAL", CAL, 2, {{  0,           0,         16}, {   0,       0,        0xFFFF}, {  0, ARG_DEF_VAL,          0}}},
    {"PCA", PCA, 3, {{  0,           0,     0xFFFF}, {   0,     0,            0x3F}, {  0,           0,      0xFFF}}},
//  {"NEW", NEW, 0, {{  MIN, Default,     MAX}, {   MIN, Default,     MAX}, {  MIN, Default,          MAX}}}
};

/************************************************************************************************
 *
 *
 ************************************************************************************************/
static int32_t validate_argument(int value, int command, int argument_index)
{
    int32_t i;

    if ((command < 0) || (argument_index < 0))
        return 0;

    for (i = 0; i < (NUMBER_OF_COMMANDS - 1); i++)
    {
        if (command_lookup[i].cmd_ID == command)
            break;
    }
    if ((i >= (NUMBER_OF_COMMANDS - 1)) || (argument_index >= command_lookup[i].number_of_arguments))
        return 0;

    if ((command_lookup[i].argument[argument_index][ARG_MIN_INDEX] > value) ||
        (command_lookup[i].argument[argument_index][ARG_MAX_INDEX] < value))
    {
        return 0;
    }

    return 1;
}

/************************************************************************************************
 *
 *
 ************************************************************************************************/
static void FSM_cleanup()
{
    CI_state = CI_STATE_CMD;
    rx_data_neg_one = '\0';
    rx_data[0] = '\0';
    rx_data[1] = '\0';
    rx_data[2] = '\0';
    rx_data[3] = '\0';
}

/************************************************************************************************
 *
 *
 ************************************************************************************************/
static void ACK_reset_UART_FSM()
{
    MTK_UartPutString("ACK\r\n");
    cmd_done_flag = 1;
    FSM_cleanup();
}

/************************************************************************************************
 *
 *
 ************************************************************************************************/
static void NAK_reset_UART_FSM()
{
    MTK_UartPutString("NAC\r\n");
    cmd_done_flag = 0;
    FSM_cleanup();
}

/************************************************************************************************
 *
 *
 ************************************************************************************************/

void UART_RX_ISR()
{
    uint8_t temp;
    int32_t i, j;
    static int32_t cmd_param_index;
    static int32_t current_arg_value;
    static bool new_arg = false, ignore_space = false;

    MTK_ClearRxInterruptSource(MTK_INTR_RX_NOT_EMPTY);
    
    temp = MTK_UartGetChar();
    if ((temp == '\0')||(temp =='\377'))
        return;

    if ((ignore_space == true) && ((temp == ' ') || (temp == '\t')))
        return;

    if (ignore_space == false)
    {
        rx_data_neg_one = rx_data[0];
        rx_data[0] = rx_data[1];
        rx_data[1] = rx_data[2];
        rx_data[2] = rx_data[3];
    }
    rx_data[3] = temp;

    if (CI_state == CI_STATE_CMD)
    {
        for (i = 0; i < (NUMBER_OF_COMMANDS - 1); i++)
        {
            for (j = 0; j < MAX_CMD_LEN; j++)
            {
                if (command_lookup[i].cmd_string[j] != rx_data[j])
                {
                    break;
                }
            }

            if ((j >= MAX_CMD_LEN) && ((rx_data_neg_one == ' ') || (rx_data_neg_one == '\n') ||
                (rx_data_neg_one == '\r') || (rx_data_neg_one == '\0') || (rx_data_neg_one == '\b')
                || (rx_data_neg_one == '\t')))
            {
                if ((command_lookup[i].number_of_arguments == 0) && ((rx_data[3] == ' ')
                                                    || (rx_data[3] == '\t')))
                {
                    ignore_space = true;
                }
                else if (((command_lookup[i].number_of_arguments > 0) && (rx_data[3] == ' ')) || 
                        (rx_data[3] == '\n') || (rx_data[3] == '\r') || (rx_data[3] == '\0'))
                {
                    cmd_done_flag = 0;
                    rx_cmd = command_lookup[i].cmd_ID;
                    rx_cmd_len = command_lookup[i].number_of_arguments;
                    cmd_index = i;
                    new_arg = false;
                    ignore_space = false;

                    if (rx_cmd_len == 0)
                    {
                        ACK_reset_UART_FSM();
                        return;
                    }
                    else if ((command_lookup[i].number_of_arguments > 0) && ((rx_data[3] == '\n') ||
                            (rx_data[3] == '\r') || (rx_data[3] == '\0')))
                    {
                        int32_t k;

                        for (k = 0; k < command_lookup[i].number_of_arguments; k++)
                        {
                            cmd_argument_array[k] = command_lookup[i].argument[k][ARG_DEF_INDEX];
                        }
                        ACK_reset_UART_FSM();
                        return;
                    }
                    else
                    {
                        new_arg = true;
                        cmd_param_index = 0;
                        CI_state = CI_STATE_DATA;
                    }
                    break;
                }
                else
                {
                    ignore_space = false;
                }
            }
            else if (j >= MAX_CMD_LEN)
            {
                NAK_reset_UART_FSM();
                return;
            }
        }
        
        if ((i >= (NUMBER_OF_COMMANDS - 1)) && ((rx_data[3] == '\n') ||
                (rx_data[3] == '\r') || (rx_data[3] == '\0')))
        {
            if ((rx_data[2] != '\r') && (rx_data[2] != '\0') && (rx_data[2] != '\n'))
            {
                NAK_reset_UART_FSM();
            }
            return;
        }
    }
    else if (CI_state == CI_STATE_DATA)
    {
        if ((temp == ' ') || (temp == '\n') || (temp == '\r') || (temp == '\0') || (temp == '\t'))
        {
            if ((new_arg == true) && (temp == ' '))
            {
                return;
            }

            if (new_arg == false)
            {
                if (negative_arg == 1)
                {
                    if (current_arg_value == 0)
                    {
                        NAK_reset_UART_FSM();
                        return;
                    }
                    current_arg_value *= -1;
                }

                if ((cmd_param_index >= MAX_CMD_ARG_LEN) || 
                    !validate_argument(current_arg_value, rx_cmd, cmd_param_index))
                {
                    NAK_reset_UART_FSM();
                    return;
                }

                cmd_argument_array[cmd_param_index] = current_arg_value;
                cmd_param_index++;
            }

            if ((rx_data[3] == '\n') || (rx_data[3] == '\r') || (rx_data[3] == '\0') || (rx_data[3] == '\t'))
            {
                if (cmd_param_index >= command_lookup[cmd_index].number_of_arguments)
                {
                    ACK_reset_UART_FSM();
                    return;
                }
                else if ((cmd_param_index < command_lookup[cmd_index].number_of_arguments))
                {
                    int32_t k;

                    for (k = cmd_param_index; k < command_lookup[cmd_index].number_of_arguments; k++)
                    {
                        cmd_argument_array[k] = command_lookup[cmd_index].argument[k][ARG_DEF_INDEX];
                    }
                    ACK_reset_UART_FSM();
                    return;
                }
            }
            new_arg = true;
            return;
        }

        if (new_arg == true)
        {
            current_arg_value = 0;
            negative_arg = 0;
        }

        if ((temp >= 0x30) && (temp <= 0x39))
        {
            int32_t temp_val;
            temp_val = current_arg_value;
            current_arg_value *= (int32_t)10;
            current_arg_value += (int32_t)(temp - 0x30);
            if (current_arg_value < temp_val)
            {
                NAK_reset_UART_FSM();
                return;
            }
        }
        else if ((new_arg == true) && (temp == '-'))
        {
            negative_arg = 1;
        }
        else if ((new_arg == true) && (temp == '+'))
        {
            return;
        }
        else/* if ((temp < 0x30) || (temp > 0x39))*/
        {
            NAK_reset_UART_FSM();
            return;
        }

        new_arg = false;
    }
    else
    {
        CI_state = CI_STATE_CMD;
    }    
}

/************************************************************************************************
 *
 *
 ************************************************************************************************/
UART_CMDS_T CI_get_command(int32_t *argument_array, int32_t *num_args)
{
    int32_t i;
    if (cmd_done_flag)
    {
        cmd_done_flag = 0;
        for (i = 0; i < rx_cmd_len; i++)
            argument_array[i] = cmd_argument_array[i];
        *num_args = rx_cmd_len;
        return rx_cmd;
    }

    return NO_CMD;
}

/************************************************************************************************
 *
 *
 ************************************************************************************************/
void MTK_CI_start()
{
    CI_state = CI_STATE_CMD;
    rx_cmd = NO_CMD;
    cmd_done_flag = 0;
    MTK_Start();
    Cy_SCB_SetRxInterruptMask(MTK_HW,MTK_INTR_RX_NOT_EMPTY);
    MTK_SetCustomInterruptHandler((cyisraddress)UART_RX_ISR);
    MTK_ClearRxInterruptSource(0xFFFF);//Clear All interrupts
}

/* [] END OF FILE */
