/* ========================================
 *
 * Copyright YOUR COMPANY, THE YEAR
 * All Rights Reserved
 * UNPUBLISHED, LICENSED SOFTWARE.
 *
 * CONFIDENTIAL AND PROPRIETARY INFORMATION
 * WHICH IS THE PROPERTY OF your company.
 * 
 * ========================================
 * This file has the code that enables Regulatory testings
 * for regulations such as FCC,IC,ETSI, KC, MIC, etc. Button 
 * press is needed to switch between various transmit and receive modes
*/

#include <project.h>
#include "TwoWire.h"
#include "RegulatoryTest.h"
#include "CyBLE_MTK.h"
#include "ble_hal.h"

#if (defined REGULATORY)

/*****************************************************************************
* Global Variable Declaration
*****************************************************************************/

DUTStates DUTState = DUT_IDLE;
#ifdef TRICOLOR_LED
LEDColors CurrentColor = LED_OFF;
uint8_t LEDToggleState = 0;
#endif
    
uint8_t CurrentChannel = 0;
uint8_t ButtonPressed = 0;
#endif

#if (defined REGULATORY)
/*******************************************************************************
* Function Name: ButtonTriggerISR
********************************************************************************
*
* Summary:
*  Interrupt Service Routine for Button presses. This routine sets a global
* variable that records the interrupt event and disbales the button interrupt.
*
* Parameters:  
*  None
*
* Return: 
*  None 
*  
*******************************************************************************/

void ButtonTriggerISR(void)
{
    ButtonPressed = 1;
    /* Diable the triggered pin interrupt */
    __NVIC_DisableIRQ(ButtonInterrupt_cfg.intrSrc);
}

#ifdef TRICOLOR_LED
/*******************************************************************************
* Function Name: SetLEDColor
********************************************************************************
*
* Summary:
*  Sets the LED color of LED3 of BLE Pioneer kit to the value passed in the 
* parameters
*
* Parameters:  
*  LEDColor. Changes the LED color to one of the following colors
* LED_WHITE, //000 - RGB
* LED_CYAN,  //001 - GB
* LED_PINK,  //010 - RB
* LED_BLUE,  //011 - B
* LED_YELLOW, //100 - RG
* LED_GREEN, //101 - G
* LED_RED,   //110 - R
* LED_OFF,   //111 - None
*
* Return: 
*  None 
*  
*******************************************************************************/
void SetLEDColor(LEDColors LEDColor)
{
    uint8 Red_LED, Blue_LED, Green_LED;
    
    if((LEDColor == LED_WHITE) || (LEDColor == LED_RED) 
    || (LEDColor == LED_PINK) || (LEDColor == LED_YELLOW))
    {
        Red_LED = 0;
    }
    else
    {
        Red_LED = 1;
    }
    if((LEDColor == LED_WHITE) || (LEDColor == LED_GREEN) 
    || (LEDColor == LED_CYAN) || (LEDColor == LED_YELLOW))
    {
        Green_LED = 0;
    }
    else
    {
        Green_LED = 1;
    }

    if((LEDColor == LED_WHITE) || (LEDColor == LED_BLUE) 
    || (LEDColor == LED_CYAN) || (LEDColor == LED_PINK))
    {
        Blue_LED = 0;
    }
    else
    {
        Blue_LED = 1;
    }
    Red_LED_Write(Red_LED);
    Green_LED_Write(Green_LED);
    Blue_LED_Write(Blue_LED);
}

/*******************************************************************************
* Function Name: ToggleLED
********************************************************************************
*
* Summary: Toggles the LED state between ON and OFF with LEDColor
* if theToggleCondition is true
* 
*
* Parameters:  
* ToggleCondition   :- A logical expression that evaluates to true or false
* LEDColor          :- LED color for blink
*
* Return: 
*  None 
*  
*******************************************************************************/

void ToggleLED(bool ToggleCondition, char LEDColor)
{
    if(ToggleCondition)
    {
        if(LEDToggleState)
        {
            LEDToggleState = 0;
            SetLEDColor(LED_OFF);
        }
        else
        {
            LEDToggleState = 1;
            SetLEDColor(LEDColor);
        }
        CyDelay(50);
    }
}
#endif
/*******************************************************************************
* Function Name: RegulatoryTestInit
********************************************************************************
*
* Summary:
*  Initialization Routine. Starts the button interrupt, assigns a ISR for the 
* interrupt and turns OFF the LED
*
* Parameters:  
*  None
*
* Return: 
*  None 
*  
*******************************************************************************/


void RegulatoryTestInit(void)
{
    Cy_SysInt_Init(&ButtonInterrupt_cfg, ButtonTriggerISR);
	NVIC_ClearPendingIRQ(ButtonInterrupt_cfg.intrSrc);
	NVIC_EnableIRQ(ButtonInterrupt_cfg.intrSrc);
#ifdef TRICOLOR_LED    
    SetLEDColor(LED_OFF);
#endif
#ifdef USE_LCD
    LCD_Start();
    LCD_PrintString("IDLE");
#endif
    //SetPower(TRANSMIT_POWER_LEVEL);
    //ll_hal_dtm_tx_test(37,2,19);

}


/*******************************************************************************
* Function Name: CycleThroughTxRxStates
********************************************************************************
*
* Summary:
* Cycles through the following states and performs corresponding action upon
* a button Press.
*   DUT_IDLE - Idle state. Does nothing
*   DUT_RX_2402 - Receiver is turned ON at RF Channel 0
*   DUT_RX_2440 - Receiver is turned ON at RF Channel 19
*   DUT_RX_2480 - Receiver is turned ON at RF Channel 39
*   DUT_TX_2402 - Transmit packets at RF Channel 0
*   DUT_TX_2440 - Transmit packets at Channel 19
*   DUT_TX_2480 - Transmit packets at Channel 39
*
* Parameters:  
*  None
*
* Return: 
*  None 
*  
*******************************************************************************/

void CycleThroughTxRxStates(void)
{
    static eDataRates data_rate = ONE_MBPS;
    //Stop the current DTM task    
    ll_hal_stop_dtm(); 
    //switch to the next state    
    if(DUT_TX_HOPPING == DUTState)
    {
        DUTState = DUT_RX_2402;  
        if (data_rate == ONE_MBPS)
        {
            data_rate = TWO_MBPS;
        }
        else
        {
            data_rate = ONE_MBPS;
        }
        CyBleHal_SetDataRate(data_rate);
    }
    else
    {
        //Reset BLE
        DUTState++;
    }
    DisplayDeviceState(DUTState);
    /*Set the LED color and change the perform the function
    appropriate to the state */
    switch(DUTState)
    {
        case DUT_IDLE:
            break;
        
        case DUT_RX_2402:
            //ss used defines for 0, 19 etc. may be you can call LOW, MID or 
            //whatever FCC calls
            CurrentChannel = 0;
            ll_hal_dtm_rx_test(CurrentChannel);
            break;
            
        case DUT_RX_2440:
            CurrentChannel = 19;
            ll_hal_dtm_rx_test(CurrentChannel);
                      
            break;
            
        case DUT_RX_2480:
            CurrentChannel = 39;
            ll_hal_dtm_rx_test(CurrentChannel);        
            break;
            
        case DUT_TX_2402:
            CurrentChannel = 0;
            ll_hal_dtm_tx_test(0xFF, DTM_PKT_TYPE_PRBS9, CurrentChannel);
            break;
            
        case DUT_TX_2440:
            CurrentChannel = 19;
            ll_hal_dtm_tx_test(0xFF, DTM_PKT_TYPE_PRBS9, CurrentChannel);
            break;
            
        case DUT_TX_2480:
            CurrentChannel = 39;
            ll_hal_dtm_tx_test(0xFF, DTM_PKT_TYPE_PRBS9, CurrentChannel);
            break;
            
        case DUT_TXC_2402:
            CurrentChannel = 0;
            CyDelay(3);
            ll_hal_transmit_carrier_wave(CurrentChannel);
            break;
        case DUT_TXC_2440:
            CurrentChannel = 19;
            ll_hal_transmit_carrier_wave(CurrentChannel);
            break;
        case DUT_TXC_2480:
            CurrentChannel = 39;
            ll_hal_transmit_carrier_wave(CurrentChannel);
            break;
        case DUT_TX_HOPPING:
            StartPseudoRandomHopping();
            break;
        default:
            DUTState = DUT_IDLE;    
            break;
            
    }
    SetStateColorCode(DUTState); 
}


int Button_Trigger_Read(void)
{
    return Cy_GPIO_Read(Button_PORT, Button_NUM);
}
/*******************************************************************************
* Function Name: CheckButtonPressStatus
********************************************************************************
*
* Summary: Handles debounce for the button press (if button is pressed)
* and returns true is a valid button press is detected.
* 
*
* Parameters:  
* None
*
* Return: 
* true: if button press is confirmed after debounce
* false: if button is not pressed or if the trigger is a bounce
*  
*******************************************************************************/

bool CheckButtonPressStatus(void)
{
    bool Status =  false;
    if(ButtonPressed)
    {
        //Clear the button pressed flag
        ButtonPressed = 0;
        //Delay for debounce
        CyDelay(100);
     	Cy_GPIO_ClearInterrupt(Button_PORT, Button_NUM);
    	NVIC_ClearPendingIRQ(ButtonInterrupt_cfg.intrSrc);
        /* Ensure the pin stays low after press
        part of debounce logic */
        if(Button_Trigger_Read())
        {
            Status = true;
        }
        //Enable the button trigger interrupt
        NVIC_EnableIRQ(ButtonInterrupt_cfg.intrSrc);
    }
    return Status;
}

/*******************************************************************************
* Function Name: Frequency Hopping Test
********************************************************************************
*
* Summary: Triggers a transmission with pseudo random hopping
* 
*
* Parameters:  
* None
*
* Return: 
* true: if button press is confirmed after debounce
* false: if button is not pressed or if the trigger is a bounce
*  
*******************************************************************************/
void StartPseudoRandomHopping(void)
{
    //SetPower(TRANSMIT_POWER_LEVEL); 
    while(1)
    {
        CurrentChannel+=9;
        if(CurrentChannel > 39)
        CurrentChannel-=40;
        ll_hal_dtm_tx_test(0x25, DTM_PKT_TYPE_PRBS9, CurrentChannel);
        CyDelayUs(1450);
        ll_hal_stop_dtm();
        CyDelay(6);
        if(CheckButtonPressStatus())
        {
#ifdef USE_LCD       
            LCD_ClearDisplay();
            LCD_Position(0,0);
            LCD_PrintString("IDLE");
#endif            
            break;
        }
    }
}

/*******************************************************************************
* Function Name: ProcessRegulatoryTestTrigger
********************************************************************************
*
* Summary:
* Handles the Button Trigger, debounce and LED Toggles
*
* Parameters:  
*  None
*
* Return: 
*  None 
*  
*******************************************************************************/

void ProcessRegulatoryTestTrigger(void)
{
    /*Toggle the LEDs for indicating burst transmission; 
    Solid glow for continuous reception*/
 
    ToggleLED((DUTState > DUT_RX_2480) && (DUTState <= DUT_TX_2480), CurrentColor);
   
    if (CheckButtonPressStatus())
    {
        CycleThroughTxRxStates();
    }
}    

#ifdef USE_LCD    
void DisplayDeviceState(DUTStates DUTState)
{
    switch(DUTState)
    {
        case DUT_IDLE:
            LCD_ClearDisplay();
            LCD_Position(0,0);
            LCD_PrintString("IDLE STATE");
            break;
            
        case DUT_RX_2402:
            LCD_ClearDisplay();
            LCD_Position(0,0);
            LCD_PrintString("RECEIVE");
            LCD_Position(1,0);
            LCD_PrintString("Freq: 2402 MHz");
            break;
            
        case DUT_RX_2440:
            LCD_ClearDisplay();
            LCD_Position(0,0);
            LCD_PrintString("RECEIVE");
            LCD_Position(1,0);
            LCD_PrintString("Freq: 2440 MHz");
            break;
            
        case DUT_RX_2480:
            LCD_ClearDisplay();
            LCD_Position(0,0);
            LCD_PrintString("RECEIVE");
            LCD_Position(1,0);
            LCD_PrintString("Freq: 2480 MHz");
            break;
            
        case DUT_TX_2402:
            LCD_ClearDisplay();
            LCD_Position(0,0);
            LCD_PrintString("DTM TRANSMIT");
            LCD_Position(1,0);
            LCD_PrintString("Freq: 2402 MHz");  
            break;
            
        case DUT_TX_2440:
            LCD_ClearDisplay();
            LCD_Position(0,0);
            LCD_PrintString("DTM TRANSMIT");
            LCD_Position(1,0);
            LCD_PrintString("Freq: 2440 MHz");  
            break;
            
        case DUT_TX_2480:
            LCD_ClearDisplay();
            LCD_Position(0,0);
            LCD_PrintString("DTM TRANSMIT");
            LCD_Position(1,0);
            LCD_PrintString("Freq: 2480 MHz");  
            break;
            
        case DUT_TXC_2402:
            LCD_ClearDisplay();
            LCD_Position(0,0);
            LCD_PrintString("CW TRANSMIT");
            LCD_Position(1,0);
            LCD_PrintString("Freq: 2402 MHz");  
            break;
            
        case DUT_TXC_2440:
            LCD_ClearDisplay();
            LCD_Position(0,0);
            LCD_PrintString("CW TRANSMIT");
            LCD_Position(1,0);
            LCD_PrintString("Freq: 2440 MHz");  
            break;            
            
        case DUT_TXC_2480:
            LCD_ClearDisplay();
            LCD_Position(0,0);
            LCD_PrintString("CW TRANSMIT");
            LCD_Position(1,0);
            LCD_PrintString("Freq: 2480 MHz");  
            break;
            
        case DUT_TX_HOPPING:
            LCD_ClearDisplay();
            LCD_Position(0,0);
            LCD_PrintString("TRANSMISSION");
            LCD_Position(1,0);
            LCD_PrintString("Freq Hopping");  
            break;
        default:
        break;
    }
}
#endif    

#ifdef TRICOLOR_LED
void SetStateColorCode(DUTStates DUTState)
{
    switch(DUTState)
    {
        case DUT_IDLE:
            CurrentColor = LED_OFF;
            break;
        case DUT_RX_2402:
            CurrentColor = LED_YELLOW;
            break;
        case DUT_RX_2440:
            CurrentColor = LED_PINK;
            break;
        case DUT_RX_2480:
            CurrentColor = LED_BLUE;
            break;
        case DUT_TX_2402:
            CurrentColor = LED_YELLOW;
            break;
        case DUT_TX_2440:
            CurrentColor = LED_PINK;
            break;
        case DUT_TXC_2402:
            CurrentColor = LED_YELLOW;
            break;
        case DUT_TXC_2440:
            CurrentColor = LED_PINK;
            break;
        case DUT_TXC_2480:
            CurrentColor = LED_BLUE;
            break;
        default:
            break;
        
    }
    SetLEDColor(CurrentColor);
}
#endif  

#endif
/* [] END OF FILE */
