/* ========================================
 *
 * Copyright YOUR COMPANY, THE YEAR
 * All Rights Reserved
 * UNPUBLISHED, LICENSED SOFTWARE.
 *
 * CONFIDENTIAL AND PROPRIETARY INFORMATION
 * WHICH IS THE PROPERTY OF your company.
 *
 * ========================================
*/
#include <stdbool.h>
#include <TwoWire.h>
#include "ble_hal.h"
#include "UART_Compatibility.h"
/*****************************************************************************
* Global Variable Declaration
*****************************************************************************/
volatile bool DTMCmdDetected = false, DTMInvalidCmd = false;
DTM_Cmd_Packet CurrentCmd = {DTM_INVALID, 0, 0, 0, 0, 0, 0};
DTM_Cmd_Packet PreviousCmd = {DTM_INVALID, 0, 0, 0, 0, 0, 0};

/*******************************************************************************
* Function Name: ISR_2_Wire_Cmd_Decoder
********************************************************************************
*
* Summary:
*  This ISR decodes the incoming 2 wire command
*
* Parameters:  
*  None
*
* Return: 
*  None 
*  
*******************************************************************************/


void ISR_2_Wire_Cmd_Decoder()
{
    static bool checkLSByte = false;
    
    if(UART_2_Wire_CHECK_INTR_RX_MASKED(UART_2_Wire_INTR_RX_NOT_EMPTY))
    {
        uint8 byte = (uint8)UART_2_Wire_SpiUartReadRxData();

        if(checkLSByte)
        {
            if((CurrentCmd.Cmd_Code == DTM_RX_TEST) || (CurrentCmd.Cmd_Code == DTM_TX_TEST))
            {
                CurrentCmd.Payload_Type = byte & PL_TYPE_MASK;
                CurrentCmd.Payload_Len = (byte & PL_LEN_MASK) >> PL_LEN_SHIFT;
                if(PreviousCmd.Cmd_Code == DTM_RESET)
                {
                    CurrentCmd.Payload_Len += PreviousCmd.Payload_Len_MSB << 6;
                }
                
                /*Both the bytes decoded, reset the decoder state*/
            }
            else
            {
                CurrentCmd.Parameter = byte >> RS_PAR_SHIFT;
                if(CurrentCmd.Cmd_Code == DTM_RESET)
                {
                    if(CurrentCmd.Control == 0x01)
                    {
                        CurrentCmd.Payload_Len_MSB = CurrentCmd.Parameter;
                    }
                }
            }
            DTMCmdDetected = true;
            checkLSByte = false;
        }
        else
        {
            CurrentCmd.Cmd_Code = (byte & CMD_CODE_MASK) >> CMD_CODE_SHIFT;
            if((CurrentCmd.Cmd_Code == DTM_RX_TEST) || (CurrentCmd.Cmd_Code == DTM_TX_TEST))
            {
                CurrentCmd.Channel = byte & CHNL_FREQ_MASK;
                if(CHNL_FREQ_LIMIT > CurrentCmd.Channel)
                {
                    /*LSByte detected, Process the MSByte*/
                    checkLSByte = true;
                }
                else
                {
                    DTMInvalidCmd = true; 
                }
            }
            else
            {
                CurrentCmd.Control = byte & RS_CON_MASK;
                checkLSByte = true;
            }
        }

        UART_2_Wire_ClearRxInterruptSource(UART_2_Wire_INTR_RX_NOT_EMPTY);
    }
}


/*******************************************************************************
* Function Name: void TwoWireInit(void)
********************************************************************************
*
* Summary:
*  Initialization Routine. Starts the 2-wire interface and assigns a ISR for the 
* 2-wire interrupt.
*
* Parameters:  
*  None
*
* Return: 
*  None 
*  
*******************************************************************************/

void TwoWireInit(void)
{
    UART_2_Wire_Start();
    Cy_SCB_SetByteMode(UART_2_Wire_HW,1);
    UART_2_Wire_HW->TX_CTRL &= ~(0x1<<8);//LSB First during TX
    UART_2_Wire_HW->RX_CTRL &= ~(0x1<<8);//LSB First during RX
    Cy_SCB_SetRxInterruptMask(UART_2_Wire_HW,UART_2_Wire_INTR_RX_NOT_EMPTY);
    UART_2_Wire_SetCustomInterruptHandler((cyisraddress)ISR_2_Wire_Cmd_Decoder);
    UART_2_Wire_ClearRxInterruptSource(0xFFFF);//Clear All interrupts
}


/*******************************************************************************
* Function Name: Process2WireCommands
********************************************************************************
*
* Summary:
*  Process the 2-wire commands.
*
* Parameters:  
*  None
*
* Return: 
*  None 
*  
*******************************************************************************/

void Process2WireCommands(void)
{
    uint8 DTM_Event[2] = {0,0};    
    if(DTMCmdDetected)
    {
        /* Process the DTM Command */
        switch(CurrentCmd.Cmd_Code)
        {
            case DTM_RESET:
                DTM_Event[1] = EVT_STATUS_SUCCESS; //SUCCESS
                DTM_Event[0] = STATUS_EVT_CODE; //event code
                CyBleHal_SetDTMMode(DTM_STOP);
                if(CurrentCmd.Control == 0x00)
                {
                    //Set RF_PHY to 1 Mbps
                    CyBleHal_SetDataRate(ONE_MBPS);
                }
                else if(CurrentCmd.Control == 0x02)
                {
                    //Set RF-PHY as per the parameter in the reset command
                    if(CurrentCmd.Parameter == 1)
                    {
                        CyBleHal_SetDataRate(ONE_MBPS);
                    }
                    else if(CurrentCmd.Parameter == 2)
                    {
                        CyBleHal_SetDataRate(TWO_MBPS);
                    }
                }
                else if(CurrentCmd.Control == 0x04)
                {
                    //send the supported test features in response
                    DTM_Event[1] |= DTM_DLE_SUPPORTED;
                    DTM_Event[1] |= DTM_2MPHY_SUPPORTED;
                }
                else if(CurrentCmd.Control == 0x05)
                {
                    switch(CurrentCmd.Parameter)
                    {
                        case READ_SUPPORTED_MAX_TX_OCTETS:
                        {
                            DTM_Event[1]|= (uint8) SUPPORTED_MAX_TX_OCTETS << 1;
                            DTM_Event[0]|= (uint8) SUPPORTED_MAX_TX_OCTETS >> 7;
                            break;
                        }
                        case READ_SUPPORTED_MAX_TX_TIME:
                        {
                            DTM_Event[1]|= (uint8) SUPPORTED_MAX_TX_TIME << 1;
                            DTM_Event[0]|= (uint8) SUPPORTED_MAX_TX_TIME >> 7;
                            break;
                        }
                        case READ_SUPPORTED_MAX_RX_OCTETS:
                        {
                            DTM_Event[1]|= (uint8) SUPPORTED_MAX_RX_OCTETS << 1;
                            DTM_Event[0]|= (uint8) SUPPORTED_MAX_RX_OCTETS >> 7;
                            break;
                        }
                        case READ_SUPPORTED_MAX_RX_TIME:
                        {
                            DTM_Event[1]|= (uint8) SUPPORTED_MAX_RX_TIME << 1;
                            DTM_Event[0]|= (uint8) SUPPORTED_MAX_RX_TIME >> 7;
                            break;
                        }
                        default:
                        break;
                    }
                }
                /*send the DTM packet status event*/
                UART_2_Wire_SpiUartPutArray(DTM_Event, sizeof(DTM_Event));
                
                break;

            case DTM_RX_TEST:

                /*send the DTM packet status event*/
                DTM_Event[1] = EVT_STATUS_SUCCESS; //SUCCESS
                DTM_Event[0] = STATUS_EVT_CODE; //event code

                UART_2_Wire_SpiUartPutArray(DTM_Event, sizeof(DTM_Event));
                CyBleHal_SetChannel(CurrentCmd.Channel);
                CyBleHal_SetDTMMode(DTM_RECEIVE);                
                break;

            case DTM_TX_TEST:
                
                /*send the DTM packet status event*/
                DTM_Event[1] = EVT_STATUS_SUCCESS; //SUCCESS
                DTM_Event[0] = STATUS_EVT_CODE; //event code

                UART_2_Wire_SpiUartPutArray(DTM_Event, sizeof(DTM_Event));
                                
                CyBleHal_SetChannel(CurrentCmd.Channel);
                //CyBleHal_DynamicCompensation(CurrentCmd.Channel);
                if(CurrentCmd.Payload_Len == 0x3F)
                {
                    CyBleHal_SetDataLength(255);
                }
                else
                {
                    CyBleHal_SetDataLength(CurrentCmd.Payload_Len);
                }
                
                CyBleHal_SetPayloadPattern(CurrentCmd.Payload_Type);
                CyBleHal_SetDTMMode(DTM_TRANSMIT);
                break;

            case DTM_END:
                {
                    uint16 value;        
                    value = CY_GET_REG32(CYREG_BLE_BLELL_DTM_RX_PKT_COUNT);                    
                    DTM_Event[1u] = (uint8) value;
                    DTM_Event[0u] = (uint8) (value >> 8u);
                    DTM_Event[0] |= REPORT_EVT_MASK; //Set the report event code
                    /*send the DTM packet Report event*/
                    UART_2_Wire_SpiUartPutArray(DTM_Event, sizeof(DTM_Event));
                    CyBleHal_SetDTMMode(DTM_STOP);
                }
                break;

            default:
                /*Invalid test*/
                /*send the DTM packet status failure event*/
                DTM_Event[1] = EVT_STATUS_FAIL; //FAIL
                DTM_Event[0] = STATUS_EVT_CODE; //event code
                UART_2_Wire_SpiUartPutArray(DTM_Event, sizeof(DTM_Event));
                break;
        }
        PreviousCmd = CurrentCmd;
        
        DTMCmdDetected = false;
        CurrentCmd.Cmd_Code = DTM_INVALID;
        CurrentCmd.Channel = 0;
        CurrentCmd.Payload_Len = 0;
        CurrentCmd.Payload_Type = 0;
        
    }
    /*send the DTM packet status failure event, if wrong command is detected*/
    if(DTMInvalidCmd)
    {
        DTMInvalidCmd = false;
        DTM_Event[1] = EVT_STATUS_FAIL; //FAIL
        DTM_Event[0] = STATUS_EVT_CODE; //event code

        UART_2_Wire_SpiUartPutArray(DTM_Event, sizeof(DTM_Event));
    }
}

/* [] END OF FILE */
