/***************************************************************************//**
* \file cy_ble_hal_pvt.h
* \version `$CY_MAJOR_VERSION`.`$CY_MINOR_VERSION`
*
* \brief
*  Contains the function prototypes and constants for the HAL section
*  of the BLE Middleware.
*
********************************************************************************
* \copyright
* Copyright 2017-2018, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#ifndef CY_BLE_HALL_PVT_H
#define CY_BLE_HALL_PVT_H

#include "cy_ble_stack.h"
#include "cy_ble_clk.h"

#include "syspm/cy_syspm.h"
#include "flash/cy_flash.h"
#include "ipc/cy_ipc_pipe.h"
#include "cy_device_headers.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/** BD Address of device */
typedef cy_stc_ble_bd_addr_t     cy_stc_ble_gap_bd_addr_t;
/***************************************
* Enumerated Types and Structures
***************************************/

typedef enum
{
    CY_BLE_IC_PACKAGE_TYPE_BGA_116 = 0ul,
    CY_BLE_IC_PACKAGE_TYPE_CSP_104,
    CY_BLE_IC_PACKAGE_TYPE_BGA_124,
    CY_BLE_IC_PACKAGE_TYPE_QFN_68
}cy_en_ble_ic_package_type_t;
/** \endcond */

/***************************************
*   HAL Constants
***************************************/

#define CY_BLE_SFLASH_BLE_RADIO_CALL_ADDRESS   ((uint32_t)SFLASH->BLE_DEVICE_ADDRESS + sizeof(cy_stc_ble_gap_bd_addr_t))

/* Flash blocking mode(define for mode in Cy_BLE_HAL_NvramWrite function )*/
#define CY_BLE_FLASH_BLOCKING_MODE           (1u)
/* Flash non blocking mode(define for mode in Cy_BLE_HAL_NvramWrite function )*/
#define CY_BLE_FLASH_NON_BLOCKING_MODE       (0u)


/* Constants for Voltage value calculation  */
#define CY_BLE_RADIO_VOLTAGE_MONITOR_SLOPE             (174u)
#define CY_BLE_RADIO_VOLTAGE_MONITOR_SLOPE_MULTIPLIER  (10u)
#define CY_BLE_RADIO_VOLTAGE_MONITOR_OFFSET            (879u)

/* Constants for Temperature value calculation  */
#define CY_BLE_RADIO_TEMP_MONITOR_SLOPE                (10315)
#define CY_BLE_RADIO_TEMP_MONITOR_SLOPE_MULTIPLIER     (10000)
#define CY_BLE_RADIO_TEMP_MONITOR_OFFSET               (189)

/***************************************
*   Function Prototypes
***************************************/
/* HAL ISR prototypes */

uint32_t Cy_BLE_HAL_GetIcPackageType(void);
bool Cy_BLE_HAL_IsDefinedTx5dbm(void);

void Cy_BLE_HAL_SimoBuckSetVoltage2(cy_en_syspm_simo_buck_voltage2_t voltage);

/* API for calculation absolute values for Temperature and Voltage monitor */
void Cy_BLE_HAL_RadioGetAbsVbat(const uint16_t *raw, uint16_t *vbat);
void Cy_BLE_HAL_RadioGetAbsTemp(const uint16_t *raw, int16_t *temp);

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* CY_BLE_HALL_PVT_H  */

/* [] END OF FILE */
