/*******************************************************************************
* File Name: reg_access.h
*
* Version: 1.0
*
* Description:
*  This file contains macros to simplify register or bit-field access.
*
* Hardware Dependency:
*  -NA-
*
********************************************************************************
* Copyright 2016, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/
#ifndef _REG_ACCESS_H_
#define _REG_ACCESS_H_

/* ============================================================================
 *      Header Files
 * ============================================================================ */

/* ============================================================================
 *      Macros
 * ============================================================================ */
/*
 * Reads a 32-bit Register.
 * Usage: ungined int temp = REG_READ32(BLE_BLELL_ADV_PARAMS);
 */
#define REG_READ32(reg)          (*((volatile unsigned int *)(reg)))

/*
 * Write a 32-bit Register.
 * Usage: REG_WRITE32(BLE_BLELL_ADV_PARAMS, 0x1234);
 */
#define REG_WRITE32(reg, val)    (REG_READ32(reg) = (val))

/*
 * Read a Bit Field in a 32-bit Register.
 * Usage: unsigned int adv_channel_map = BF_READ(BLE_BLELL_ADV_PARAMS, ADV_CHANNEL_MAP);
 */
#define BF_READ(reg, bf)         (((REG_READ32(reg)) & (reg##_##bf##_MSK)) >> (reg##_##bf##_POS))

/*
 * Write a Bit Field in a 32-bit Register.
 * Usage: BF_WRITE(BLE_BLELL_ADV_PARAMS, ADV_CHANNEL_MAP, 0x2);
 */
#define BF_WRITE(reg, bf, val)   REG_WRITE32(reg, ((REG_READ32(reg)) & (~(reg##_##bf##_MSK))) | \
                                ((((unsigned int)(val)) << (reg##_##bf##_POS)) & (reg##_##bf##_MSK)))

/*
 * Set a Bit Field in 32-bit register.
 * Usage: BF_SET(BLE_RCB_INTR, RCB_DONE)
 */
#define BF_SET(reg, bf)         REG_WRITE32(reg, ((REG_READ32(reg)) | (reg##_##bf##_MSK)))

/*
 * Clear a Bit Field in 32-bit register.
 * Usage: BF_SET(BLE_RCB_INTR, RCB_DONE)
 */
#define BF_CLR(reg, bf)         REG_WRITE32(reg, ((REG_READ32(reg)) & (~(reg##_##bf##_MSK))))

#endif /* _REG_ACCESS_H_ */

